package com.virjar.g4proxy_new.server.client;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;
import com.virjar.g4proxy_new.protocol.Constant;
import com.virjar.g4proxy_new.server.AvailablePortResManager;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class ClientManager {
    private Map<String, NatClientImage> natClientImageMap = new ConcurrentHashMap<>();
    private BiMap<String, Integer> clientPortBiMap = HashBiMap.create();
    @Setter
    private ServerBootstrap userMappingBootstrap;

    public ClientManager() {
        // this.userMappingBootstrap = userMappingBootstrap;
    }

    public boolean registerNewClient(String client, Channel channel) {
        NatClientImage natClientImage = natClientImageMap.get(client);
        if (natClientImage != null) {
            natClientImage.updateChannel(channel);
            return true;
        }

        Integer mappingPort = AvailablePortResManager.allocate();
        if (mappingPort == null) {
            log.error("failed to allocate port");
            return false;
        }

        ChannelFuture channelFuture = userMappingBootstrap.bind(mappingPort);
        try {
            channelFuture.get();
        } catch (Exception e) {
            log.error("wait for port binding error", e);
            return false;
        }
        if (!channelFuture.isSuccess()) {
            log.warn("bind mapping port:{} failed", mappingPort, channelFuture.cause());
            return false;
        }
        channel.attr(Constant.NAT_CHANNEL_CLIENT_KEY).set(client);
        clientPortBiMap.put(client, mappingPort);
        natClientImage = new NatClientImage(client, mappingPort, channel);
        natClientImageMap.put(client, natClientImage);
        log.info("client :{} register success,with port:{}", client, mappingPort);
        return true;
    }

    public NatClientImage getClient(String clientId) {
        return natClientImageMap.get(clientId);
    }

    public NatClientImage getClient(Integer mappingPort) {
        return natClientImageMap.get(clientPortBiMap.inverse().get(mappingPort));
    }


    public void closeClient(String clientId) {
        NatClientImage client = natClientImageMap.remove(clientId);
        if (client == null) {
            log.error("now client registered for clientId:{}", clientId);
            return;
        }
        client.closeAllUserChannel();
        client.getNatChannel().close();
    }
}
